/*
 * xfilesel. Portions by Maxim Chirkov <mc@tyumen.ru>
/*

/*
 * $XConsortium: xcutsel.c /main/21 1996/02/02 14:26:48 kaleb $
 *
 * 
Copyright (c) 1989  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.
 * *
 * Author:  Ralph Swick, DEC/Project Athena
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>

#include <X11/Xmu/Atoms.h>
#include <X11/Xmu/StdSel.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xfuncs.h>

#ifdef XKB
#include <X11/extensions/XKBbells.h>
#endif
#define CLIPBOARD_SIZE 16384

static XrmOptionDescRec optionDesc[] = {
    {"-selection", "selection",	XrmoptionSepArg, NULL},
    {"-select",    "selection",	XrmoptionSepArg, NULL},
    {"-sel",	   "selection",	XrmoptionSepArg, NULL},
    {"-s",	   "selection",	XrmoptionSepArg, NULL},
    {"-file", "file",	XrmoptionSepArg, NULL},
    {"-f", "file",	XrmoptionSepArg, NULL}
};

typedef struct {
    String  selection_name;
    String  file_name;
    int	    buffer;
    Atom    selection;
    char*   value;
    int     length;
} OptionsRec;

OptionsRec options;

#define Offset(field) XtOffsetOf(OptionsRec, field)

static XtResource resources[] = {
    {"selection", "Selection", XtRString, sizeof(String),
       Offset(selection_name), XtRString, "PRIMARY"},
    {"file", "File", XtRString, sizeof(int),
       Offset(file_name), XtRString, "default"}
};

#undef Offset

typedef struct {
    Widget button;
    Boolean is_on;
} ButtonState;

static ButtonState state;

Syntax(call)
	char *call;
{
    fprintf (stderr, "usage:  %s [-f name] [-selection name]\n", 
	     call);
    fprintf (stderr, "\t-f name - path of clipboard file (default value: ~/.clipboard),\n\t\tfor example: -f /dev/tty1, -f /home/mc/.cedit/cooledit.clip\n"); 
    fprintf (stderr, "\t-selection name - name of the selection to use. The default is PRIMARY.\n"); 
    fprintf (stderr, "Based on xcutsel program. Portions by Maxim Chirkov.\n\n"); 

    exit (1);
}


static void StoreBuffer(w, client_data, selection, type, value, length, format)
    Widget w;
    XtPointer client_data;
    Atom *selection, *type;
    XtPointer value;
    unsigned long *length;
    int *format;
{
    FILE * clip_file;
    char * home_env;
    char * file_name;

    if (*type == 0 || *type == XT_CONVERT_FAIL || *length == 0) {
#ifdef XKB
	XkbStdBell( XtDisplay(w), XtWindow(w), 0, XkbBI_MinorError );
#else
	XBell( XtDisplay(w), 0 );
#endif
	return;
    }

    if (strcmp(options.file_name,"default") ){
	clip_file = fopen(options.file_name,"w");
    } else {
	home_env = getenv("HOME");
	file_name = XtMalloc(strlen(home_env)+15);
	sprintf(file_name,"%s/.clipboard",home_env);
	clip_file = fopen(file_name,"w");
	XtFree(file_name);
    }
    if (clip_file == NULL){
	fprintf(stderr,"Can't open clipboard file.");
	XtFree(value);
	return;
    }
    fwrite((char*)value, (int)(*length), 1, clip_file );
    fclose(clip_file);
  
    XtFree(value);
}


static Boolean ConvertSelection(w, selection, target,
				type, value, length, format)
    Widget w;
    Atom *selection, *target, *type;
    XtPointer *value;
    unsigned long *length;
    int *format;
{
    FILE * clip_file;
    char * buffer;
    char * home_env;
    char * file_name;

    Display* d = XtDisplay(w);
    XSelectionRequestEvent* req =
	XtGetSelectionRequest(w, *selection, (XtRequestId)NULL);

    
    if (*target == XA_STRING || *target == XA_TEXT(d)) {
	*type = XA_STRING;

	if (strcmp(options.file_name,"default") ){
	    clip_file = fopen(options.file_name,"r");
	} else {
	    home_env = getenv("HOME");
	    file_name = XtMalloc(strlen(home_env)+15);
	    sprintf(file_name,"%s/.clipboard",home_env);
	    clip_file = fopen(file_name,"r");
	    XtFree(file_name);
	}
	if (clip_file == NULL){
	    fprintf(stderr,"Can't open clipboard file.");
	    return False;
	}

	buffer = XtMalloc(CLIPBOARD_SIZE);
	*length = fread(buffer, 1, CLIPBOARD_SIZE, clip_file );
	fclose(clip_file);

	*value = XtMalloc((Cardinal) *length);

	memmove( (char *) *value, buffer, *length);
	XtFree(buffer);

	*format = 8;
	return True;
    }

    if (XmuConvertStandardSelection(w, req->time, selection, target, type,
				    (XPointer *)value, length, format))
	return True;

    /* else */
    return False;
}


static void SetButton(state, on)
    ButtonState *state;
    Boolean on;
{
    if (state->is_on != on) {
	Arg args[2];
	Pixel fg, bg;
	XtSetArg( args[0], XtNforeground, &fg );
	XtSetArg( args[1], XtNbackground, &bg );
	XtGetValues( state->button, args, TWO );
	args[0].value = (XtArgVal)bg;
	args[1].value = (XtArgVal)fg;
	XtSetValues( state->button, args, TWO );
	state->is_on = on;
    }
}


static void LoseSelection(w, selection)
    Widget w;
    Atom *selection;
{
    if (options.value) {
	XFree( options.value );
	options.value = NULL;
    }
    SetButton(&state, False);
}


/* ARGSUSED */
static void Quit(w, closure, callData)
    Widget w;
    XtPointer closure;		/* unused */
    XtPointer callData;		/* unused */
{
    XtCloseDisplay( XtDisplay(w) );
    exit(0);
}


/* ARGSUSED */
static void GetSelection(w, closure, callData)
    Widget w;
    XtPointer closure;		/* unused */
    XtPointer callData;		/* unused */
{
    XtGetSelectionValue(w, options.selection, XA_STRING,
			StoreBuffer, NULL,
			XtLastTimestampProcessed(XtDisplay(w)));
}


/* ARGSUSED */
static void GetBuffer(w, closure, callData)
    Widget w;
    XtPointer closure;
    XtPointer callData;		/* unused */
{
    if (options.value) XFree( options.value );
    options.value =
	XFetchBuffer(XtDisplay(w), &options.length, options.buffer);
    if (options.value != NULL) {
	if (XtOwnSelection(w, options.selection,
			   XtLastTimestampProcessed(XtDisplay(w)),
			   ConvertSelection, LoseSelection, NULL))
	    SetButton((ButtonState*)closure, True);
    }
}

void main(argc, argv)
    int argc;
    char *argv[];
{
    char label[100];
    Widget box, button;
    XtAppContext appcon;
    Widget shell;
    XrmDatabase rdb;

    XtSetLanguageProc(NULL, NULL, NULL);

    shell =
	XtAppInitialize( &appcon, "XCutsel", optionDesc, XtNumber(optionDesc),
			 &argc, argv, NULL, NULL, 0 );

    rdb = XtDatabase(XtDisplay(shell));

    if (argc != 1) Syntax(argv[0]);

    XtGetApplicationResources( shell, (XtPointer)&options,
			       resources, XtNumber(resources),
			       NULL, ZERO );
			       
			       
			       printf( "%s",options.file_name);
    options.value = NULL;
    XmuInternStrings( XtDisplay(shell), &options.selection_name, ONE,
		      &options.selection );

    box = XtCreateManagedWidget("box", boxWidgetClass, shell, NULL, ZERO);
        
	button =
	    XtCreateManagedWidget("quit", commandWidgetClass, box, NULL, ZERO);
	    XtAddCallback( button, XtNcallback, Quit, NULL );

	snprintf(label,100, "*label:copy %s to file",options.selection_name);
	XrmPutLineResource( &rdb, label );

	button =
	    XtCreateManagedWidget("sel-cut", commandWidgetClass, box, NULL, ZERO);
	    XtAddCallback( button, XtNcallback, GetSelection, NULL );

	snprintf(label,100, "*label:copy file to %s", options.selection_name);
	XrmPutLineResource( &rdb, label );

	button =
	    XtCreateManagedWidget("cut-sel", commandWidgetClass, box, NULL, ZERO);
	    XtAddCallback( button, XtNcallback, GetBuffer, (XtPointer)&state );
 	    state.button = button;
	    state.is_on = False;
	    
	XtRealizeWidget(shell);
	XtAppMainLoop(appcon);
}
